@@ -91,7 +91,7 @@
 
 
     ## check that 'file' is valid, and 'chdir' if required
-    if (!is.character(file) || length(file) != 1L) {
+    if (!.IS_SCALAR_STR(file)) {
         stop(gettextf("'%s' must be a character string", "file", domain = "R"), domain = NA)
     } else if (grepl("^(ftp|ftps|http|https)://", file)) {
         stop("cannot '.build.this' on a URL")
@@ -577,3 +577,24 @@ vapply(files, function(file) paste0(readLines(file), "\n", collapse = ""), "")
 
 .isfalse <- function (x)
 .External2(.C_isfalse, x)
+
+
+.asInteger <- function (x)
+.External2(.C_asInteger, x)
+
+
+.asIntegerGE0 <- function (x)
+.External2(.C_asIntegerGE0, x)
+
+
+.IS_SCALAR_STR <- function (x)
+.External2(.C_IS_SCALAR_STR, x)
+
+
+.scalar_streql <- function (e1, e2)
+.External2(.C_scalar_streql, e1, e2)
+
+
+.get.dyn <- function (x, ifnotfound = stop(gettextf("object '%s' not found", as.character(x), domain = "R"), domain = NA),
+    minframe = 1L, inherits = FALSE)
+.External2(.C_get.dyn, x, minframe, inherits)
