@@ -5,10 +5,10 @@
 #ifdef ENABLE_NLS
 #include <libintl.h>
 #define _(String) dgettext ("R", String)
-#define _RGui(String) dgettext ("RGui", String)
+#define dgettext_RGui(String) dgettext ("RGui", String)
 #else
 #define _(String) (String)
-#define _RGui(String) (String)
+#define dgettext_RGui(String) (String)
 #endif
 
 
